import unittest
import string
from ..src.idgeneratorha import (
    generate_password,
    generate_guid,
    generate_credit_card_number,
    generate_object_id,
    generate_pin_number,
)
from ..src.utils import luhn_checksum


class GeneratorTest(unittest.TestCase):
    def test_generate_password(self) -> None:
        pwd = generate_password(length=6)
        self.assertTrue(
            (any(char.isupper() for char in pwd))
            and (any(char.islower() for char in pwd))
            and (any(char in string.punctuation for char in pwd))
            and (any(char in string.digits for char in pwd))
        )
        self.assertRaises(AssertionError, generate_password, length=5)

    def test_generate_guid(self) -> None:
        guid = generate_guid()
        parts = guid.split("-")
        self.assertTrue(len(parts) == 5)
        criteria = [char in string.hexdigits for part in parts for char in part]
        self.assertTrue(all(criteria))
        self.assertTrue(len(parts[0]) == 8)
        self.assertTrue(len(parts[1]) == 4)
        self.assertTrue(len(parts[2]) == 4)
        self.assertTrue(len(parts[3]) == 4)
        self.assertTrue(len(parts[4]) == 12)

    def test_generate_credit_card_number(self) -> None:
        number = generate_credit_card_number()
        self.assertTrue(luhn_checksum(number))

    def test_generate_pin_number(self) -> None:
        self.assertTrue(len(generate_pin_number(length=4)) == 4)
        self.assertTrue(
            all(digits in string.digits for digits in generate_pin_number(length=4))
        )

    def test_generate_object_id(self) -> None:
        objid = generate_object_id()
        self.assertTrue(len(objid) == 24)
        self.assertTrue(all(char in string.hexdigits for char in objid))



import random
import string
import secrets  # To generate cryptographic safe passwords
import uuid
from .utils import luhn_checksum
import bson


def generate_password(length: int = 6) -> str:
    """
    A generator that creates a password of at least 6 characters and contains
    at least one uppercase & lowercase letter, 1 number and at 1 symbol

    """
    assert length >= 6, f"Password length can't be less than 6, {length} given!"
    while True:
        pwd = ""
        for i in range(length):
            pwd += "".join(
                secrets.choice(
                    string.ascii_letters + string.digits + string.punctuation
                )
            )
        if (
            (any(char.isupper() for char in pwd))
            and (any(char.islower() for char in pwd))
            and (any(char in string.punctuation for char in pwd))
            and (any(char in string.digits for char in pwd))
        ):
            break

    return pwd


def generate_guid() -> str:
    """
    A generator that creates a GUID.

    """
    guid = str(uuid.uuid4())
    guid = guid.upper()
    return guid


def generate_credit_card_number(length: int = 8) -> str:
    """
    A credit card number generator that uses the Luhn checksum test.

    """
    number = "".join(random.choices(string.digits, k=length))
    while not luhn_checksum(number):
        number = "".join(random.choices(string.digits, k=length))
    return number


def generate_pin_number(length: int = 4) -> str:
    """
    A credit/debit card pin generator. The pin number can only contain digits.

    """
    return "".join(random.choices(string.digits, k=length))


def generate_object_id() -> str:
    """
    A generator that creates a mongodb like objectID.
    An ObjectId is a 12-byte unique identifier consisting of:

          - a 4-byte value representing the seconds since the Unix epoch,
          - a 3-byte machine identifier,
          - a 2-byte process id, and
          - a 3-byte counter, starting with a random value.

    """
    return str(bson.ObjectId())



def digits_of(number_sequence: str) -> list[int]:
    return [int(digit) for digit in str(number_sequence)]


def luhn_checksum(card_number: str) -> bool:
    digits = digits_of(card_number)
    odd_digits = digits[-1::-2]
    even_digits = digits[-2::-2]
    checksum = 0
    checksum += sum(odd_digits)
    for d in even_digits:
        checksum += sum(digits_of(str(d * 2)))
    return checksum % 10 == 0
